<?php 


class ACB
{
    public $clientId = 'iuSuHYVufIUuNIREV0FB9EoLn9kHsDbm';
    private $connect;
    private $URL = array(
        "LOGIN" => "https://apiapp.acb.com.vn/mb/auth/tokens",
        "getBalance" => "https://apiapp.acb.com.vn/mb/legacy/ss/cs/bankservice/transfers/list/account-payment",
        "INFO" => "https://mobile.mbbank.com.vn/retail_lite/loan/getUserInfo",
        "GET_TOKEN" => "https://mobile.mbbank.com.vn/retail_lite/loyal/getToken",
        "GET_NOTI" => "https://mobile.mbbank.com.vn/retail_lite/notification/getNotificationDataList",
        "GET_TRANS" => "https://apiapp.acb.com.vn/mb/legacy/ss/cs/bankservice/saving/tx-history?maxRows=20&account=4650511",
    );
    protected $username;
    protected $password;
    protected $res = '';
    protected $accessToken = '';

    public function __construct($username,$password,$account_number)
    {
        $this->file = "data/$username.txt";
        $this->password = $password;
        if(!file_exists($this->file)){
            $this->username = $username;
            $this->password = $password;
            $this->account_number = $account_number;
            $this->saveData();
        }
        else
            $this->parseData();
    }

    public function saveData(){
        $data = [
            'username'              => $this->username,
            'password'              => $this->password,
            'account_number'        => $this->account_number,
            'accessToken'           => $this->accessToken,
            'res'                   => $this->res,
        ];
        file_put_contents($this->file, json_encode($data));
    }
    public function parseData(){
        $data = file_get_contents($this->file);
        $data = json_decode($data);
        $this->username = $data->username;
        $this->password = $this->password;
        $this->account_number = isset($data->account_number) ? $data->account_number : '';
        $this->accessToken = isset($data->accessToken) ? $data->accessToken : '';
        $this->res = isset($data->res) ? $data->res : '';
    
    }

    public function headerDefault($token = ''){
        $header = array(
            'Content-Type: application/json; charset=utf-8',
            'Host: apiapp.acb.com.vn',
            'User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.26 Safari/537.36 Edg/85.0.564.13',
        );
        if($token)
            $header[] = "Authorization: bearer $token";
        return $header;
    }

    public function login(){
        $data = [
            'clientId' => $this->clientId,
            'username' => $this->username,
            'password' => $this->password
        ];
        $res = $this->CURL("LOGIN", $this->headerDefault(), $data);
        if(isset($res['accessToken'])){
            $this->res= $res;
            $this->accessToken= $res['accessToken'];
            $this->saveData();
        }
        return $res;
    }

    // public function transferMoney($accountNo, $bankNumber,$bankCode,$amount,$message){
    //     $res = $this->getBankInfo($bankCode, $bankNumber, $accountNo);
    //     if(isset($res['data']['ownerName'])){
    //         $res = $this->verifyTransfer($accountNo, $res['data'], $amount, $message);
    //         if(isset($res['data']['uuid'])){
    //             $uuid = $res['data']['uuid'];
    //         }
    //     }

    //     return $res;
    // }


    public function confirmTransfer($uuid, $code)
    {
      $result  = $this->CURL2('https://apiapp.acb.com.vn/mb/legacy/ss/cs/bankservice/transfers/verify',$this->headerDefault($this->accessToken), [
        'uuid'  =>  $uuid,
        'code'  =>  $code,
        'authMethod'  => 'OTPA',
      ]);
      return $result;
    }

    public function verifyTransfer($accountNo, $infoTarget, $amount, $text = '', $bankCode)
    {
      $ownerName  = $infoTarget['ownerName'];
      $account  = $infoTarget['account'];
      $bank  = $infoTarget['bank'];

      $result  = $this->CURL2("https://apiapp.acb.com.vn/mb/legacy/ss/cs/bankservice/transfers/submit",$this->headerDefault($this->accessToken), [
        'type' => $bankCode == '970416' ? 1 : 3,
        'authMethod' => 'OTPA',
        'menu' => '34',
        'amount' => $amount,
        'currency' => 'VND',
        'fromAccount' => $accountNo,
        'transactionAmount' => $amount,
        'receiverName' => $ownerName,
        'bankName' => '',
        'comment' => $text,
        'transferTime' => [
          'type' => 1,
          'period' => 0,
          'startDate' => 0,
          'endDate' => 0,
        ],
        'fee' => 0,
        'feeTo' => 0,
        'resultToEmails' => [],
        'accountInfo' => [
          'accountNumber' => $account,
          'bankCode' => '',
          'bankName' => '',
          'napasBankCode' => $bank,
          'bankcheckId' => 0,
        ],
        'bankCode' => '',
        'napasBankCode' => '',
        'referenceNumber' => '',
        'province' => '',
        'mbTransactionLimitInfo' => [
          'byPass' => 0,
          'bySMS' => 0,
          'byToken' => 0,
          'bySafeKey' => 0,
          'byAdvSafeKey' => '',
        ]
      ]);
      
      
      return $result;
    }

    public function getTransactions($accountNo, $rows = 10) {
        $header = $this->headerDefault($this->accessToken);
        $result = $this->CURL2("https://apiapp.acb.com.vn/mb/legacy/ss/cs/bankservice/saving/tx-history?maxRows=".$rows."&account=".$accountNo."", $header,$data = null);
        return $result;
    }

    public function getBalance() {
        $result = $this->CURL("getBalance", $this->headerDefault($this->accessToken),$data = null);
        return $result;
    }

    public function getBankInfo($bankCode, $bankNumber, $accountNo){
        $result = $this->CURL2("https://apiapp.acb.com.vn/mb/legacy/ss/cs/bankservice/transfers/accounts/{$bankNumber}?bankCode={$bankCode}&accountNumber={$accountNo}", $this->headerDefault($this->accessToken),$data = null);
        return $result;
    }

    private function CURL($Action, $header, $data)
    {
        $Data = is_array($data) ? json_encode($data) : $data;
        $curl = curl_init();
        $header[] = 'Content-Type: application/json; charset=utf-8';
        $header[] = 'accept: application/json';
        $header[] = 'Content-Length: ' . strlen($Data);
        $opt = array(
            CURLOPT_URL => $this->URL[$Action],
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => empty($data) ? false : true,
            CURLOPT_POSTFIELDS => $Data,
            CURLOPT_CUSTOMREQUEST => empty($data) ? 'GET' : 'POST',
            CURLOPT_HTTPHEADER => $header,
            CURLOPT_ENCODING => "",
            CURLOPT_HEADER => false,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_TIMEOUT => 20,
        );
        curl_setopt_array($curl, $opt);
        $body = curl_exec($curl);
        if (is_object(json_decode($body))) {
            return json_decode($body, true);
        }
        return json_decode($body, true);
    }

    public function CURL2($Action, $header, $data)
    {
        $Data = is_array($data) ? json_encode($data) : $data;
        $curl = curl_init();
        $opt = array(
            CURLOPT_URL => $Action,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => empty($data) ? false : true,
            CURLOPT_POSTFIELDS => $Data,
            CURLOPT_CUSTOMREQUEST => empty($data) ? 'GET' : 'POST',
            CURLOPT_HTTPHEADER => $header,
            CURLOPT_ENCODING => "",
            CURLOPT_HEADER => false,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_2,
            CURLOPT_TIMEOUT => 20,
        );
        curl_setopt_array($curl, $opt);
        $body = curl_exec($curl);
        if (is_object(json_decode($body))) {
            return json_decode($body, true);
        }
        return $body;
    }


}

 ?>